#define format
///format(str, arg...)
/**
 * format :: String -> ... -> String
 *
 * Formats a string by replacing parameters `%` by the given values.
 * A parameter is either an escape sequence or a conversion type.
 *
 * Available escape sequences:
 *  - `%%` for a percent sign `%`
 *  - `%'` for a double quote `"`
 *  - `%"` for a single quote `'`
 *  - `%t` for a tab character
 *  - `%n` for a newline
 *
 * Available parameter conversions:
 *  - `%s` for a string
 *  - `%i` for a decimal integer
 *  - `%x` for a hexadecimal integer
 *  - `%f` for a real value
 *  - `%o` for an instance or object
 *  - `%c` for a HTML colour code (`#RRGGBB`)
 *  - `%dm` for a ds_map
 *
 * @param str the format string
 * @param arg any value that is compatible with its respective conversion format
 * @returns a formatted string
 */

var str = argument[0];

var in = str;
var out = "";
var i = 1;

for (var pos = string_pos("%", in); pos > 0; pos = string_pos("%", in)) {
  out += string_copy(in, 1, pos-1);
  var token = _format_get_token(string_delete(in, 1, pos));
  in = string_delete(in, 1, pos+string_length(token));
  
  if (token == "")
    show_error("Unexpected format parameter.", false);
  else
    switch (token) {
    case "%":
      out += "%";
      break;
    case "'":
      out += '"';
      break;
    case '"':
      out += "'";
      break;
    case "n":
      out += chr($A);
      break;
    case "t":
      out += chr($9);
      break;
    case "s":
      out += string(argument[i]);
      i++;
      break;
    case "i":
      out += string(floor(argument[i]));
      i++;
      break;
    case "x":
      out += string_base(floor(argument[i]), 16);
      i++;
      break;
    case "f":
      out += string(argument[i]);
      i++;
      break;
    case "o":
      out += getInstObjName(argument[i]);
      i++;
      break;
    case "c":
      var r = ((argument[i] >>  0) mod 256) << 16;
      var g = ((argument[i] >>  8) mod 256) <<  8;
      var b = ((argument[i] >> 16) mod 256) <<  0;

      out += "#"+string_base(r+g+b, 16);
      i++;
      break;
    default:
      show_error("Unexpected format parameter '%"+token+"'.", false);
    }
}


out += in;

return out;


#define _format_get_token
if (string_pos(string_char_at(argument0, 1), "%'nt"+'"') > 0)
  return string_copy(argument0, 1, 1);
if (string_pos(string_char_at(argument0, 1), "sixfoc") > 0)
  return string_copy(argument0, 1, 1);
if (string_pos(string_char_at(argument0, 1), "d") > 0)
  return string_copy(argument0, 1, 2);

return "";